module OrientDb
  class CreateRelationsHelper

    include RepositoriesProxy
    include Log::Logging

    attr_accessor :subjects, :attributes, :institutions

    def initialize
      @subjects = {}
      @attributes = {}
      @institutions = {}
    end

    # create_first parameter:
    # Define the behavior when it cant find the vertex in the hash
    # True = Try to create in the database first
    # False = Try to find it in the database first

    def get_institution(name, create_first=true)
      institution = @institutions[name]
      if institution.nil?
        # Create a institution with this name if it doesnt exist
        unless create_first
          institution = institution_repository.find_by_name(name)
          unless institution.nil?
            @institutions[name] = institution
            return institution
          end
        end
        begin
          institution = Institution.new(:name => name)
          institution_repository.create(institution)
          logger.create "Institution: #{institution.id.to_s} created"
        rescue Orientdb4r::ServerError => e
          if e.message.include? "ORecordDuplicatedException"
            # Should not enter here
            # Look for institution with the same name
            institution = institution_repository.find_by_name(name)
          else
            raise e
          end
        end
        @institutions[name] = institution
      end
      institution
    end

    def get_subject(name, create_first=true)
      subject = @subjects[name]
      if subject.nil?
        # Create a Subject with this name if it doesnt exist
        unless create_first
          subject = subject_repository.find_by_name(name)
          unless subject.nil?
            subject.supertopics=[]
            @subjects[name] = subject
            return subject
          end
        end
        begin
          date = Time.now()
          subject = Subject.new(:name => name, :created_at => date, :last_modified => date)
          subject_repository.create(subject)
          logger.create "Subject: #{subject.id.to_s} created"
        rescue Orientdb4r::ServerError => e
          if e.message.include? "ORecordDuplicatedException"
            # Should not enter here
            # Look for subject with the same name
            subject = subject_repository.find_by_name(name)
          else
            raise e
          end
        end
        subject.supertopics=[]
        @subjects[name] = subject
      end
      subject
    end

    def get_attribute(key, value, create_first=true)
      attribute = @attributes[key+value]
      if attribute.nil?
        # Create an Attribute with this key and value if it doesnt exist
        unless create_first
          attribute = attribute_repository.find_by_key_and_value(key,value)
          unless attribute.nil?
            @attributes[key+value] = attribute
            return attribute
          end
        end
        begin
          attribute = Attribute.new(:key => key, :value => value)
          attribute_repository.create(attribute)
          logger.create "Attribute: #{attribute.id.to_s} created"
        rescue Orientdb4r::ServerError => e
          if e.message.include? "ORecordDuplicatedException"
            # Should not enter here
            # Look for attribute with the same key and value
            attribute = attribute_repository.find_by_key_and_value(key,value)
          else
            raise e
          end
        end
        @attributes[key+value] = attribute
      end
      attribute
    end

    # For each unique key on LearningObject metadata, get all its values and add on a new hash
    def get_unique_metadata_keys(lo)
      hash = {}
      lo.metadata.each do |m|
        unless hash.has_key?(m['key'])
          hash[m['key']] = lo.get_metadata_values_of(m['key'])
        end
      end
      hash
    end
  end
end