require 'test_helper'

class OrientDb::CollectionRepositoryTest < ActiveSupport::TestCase

  test 'raise an exception when try to save non-persisted collection' do
    odb_client = mock
    collection_repository = OrientDb::CollectionRepository.new odb_client

    assert_raise NotPersistedRecordError do
      collection_repository.save_learning_objects Collection.new
    end
  end

  test 'save learning objects to persisted collection' do
    lo = ::LearningObject.new(id: '#14:14')
    collection = ::Collection.new(id: '#12:12', learning_objects: [lo])

    # setup orient4r mock
    expected_remove_mock_arg = sprintf('update %s REMOVE learning_objects', collection.id)
    expected_add_mock_arg = sprintf("update %s ADD learning_objects = [%s] SET last_modified='%s'", collection.id, collection.learning_objects.map(&:id).join(','), Time.now())
    odb_client = mock
    odb_client.expect :command, nil, [expected_remove_mock_arg]
    odb_client.expect :command, nil, [expected_add_mock_arg]

    collection_repository = OrientDb::CollectionRepository.new odb_client
    collection_repository.save_learning_objects collection
    assert mock.verify
  end

end